package Risk.Internals;

import Risk.Internals.Exceptions.*;

/**
* Element territori que conte una llista amb tots els altres territoris amb els
* que fa frontera. El seu nom, el seu propietari, el nombre d'armies, el seu id i
* l'ID del continent al que pertany.
* A ms cont mtodes especial tils per porta a terme accions concretes de l'API
* que s'ha d'oferir als jugadors.
*/
public class Territory {
  private Llista border;
  private String nom;
  private int owner;
  private int armies;
  private int continentID;
  private int id;

  public Territory(String s, int id, int continentID) throws NoMemoria {
    nom = s;
    border = new Llista();
    this.continentID = continentID;
    owner=0;
    armies=0;
    this.id = id;
  }

  public int id(){
    return id;
  }
  
  public String nom(){
    return nom;
  }

  public int continentID(){
    return continentID;
  }
  
  public int owner(){
    return owner;
  }
  
  public void owner(int o){
    owner = o;
  }
  
  public int armies(){
    return armies;
  }
  
  public void armies(int a){
    armies = a;
  }
  
  /**
  * Afegir un nou territori que fa frontera amb aquest
  */
  public void borderWith(Territory t) throws PDIdreta, NoMemoria{
    border.anarAlFinal();
    border.insertar(t);
  }
  
  /**
  * Retorna True si aquest territori fa frontera amb l'indicat
  * @param s Territori amb el que volem comprovar si fa frontera
  * @return True aquest territori fa frontera amb l'indicat
  */
  public boolean borderWith(String s) throws PDIdreta{
    Territory t;
    
    if (border.buida()) {
      return false;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if (s.equals(t.nom())) {
        return true;
      }
      border.avanzar();
    } while (! (border.finalLlista()));
    
    return false;
  }
  
  /**
  * @return True si este territorio tiene alrededor algun territorio enemigo
  */
  public boolean isFront() throws PDIdreta{
    Territory t;
    
    if (border.buida()) {
      return false;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if ((t.owner() !=  owner) && (t.owner() != 0)) {
        return true;
      }
      border.avanzar();
    } while (! (border.finalLlista()));
    
    return false;
  }

  /**
  * @return Nombre de territoris que fan frontera amb aquest i sn de l'enemic
  */
  public int FrontsCount()  throws PDIdreta{
    Territory t;
    int total = 0;
    
    if (border.buida()) {
      return 0;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if ((t.owner() !=  owner) && (t.owner() != 0)) {
        total++;
      }
      border.avanzar();
    } while (! (border.finalLlista()));
    
    return total;
  }
  
  
  /**
  * @return 0 si no hi ha frontera o l'id del territori amb ms armies enemigues
  */
  public int StrongestFront()  throws PDIdreta{
    Territory t;
    int strongest = 0;
    int strongestArmy = 0;
    
    if (border.buida()) {
      return 0;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if ((t.owner() !=  owner) && (t.owner() != 0) && (t.armies() > strongestArmy)) {
        strongest = t.id();
        strongestArmy = t.armies();
      }
      border.avanzar();
    } while (! (border.finalLlista()));
    
    return strongest;
  }
  
  /**
  * @return 0 si no hi ha frontera o el nombre d'armies del territori amb ms armies enemigues
  */
  public int StrongestFrontArmy()  throws PDIdreta{
    Territory t;
    int strongest = 0;
    int strongestArmy = 0;
    
    if (border.buida()) {
      return 0;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if ((t.owner() !=  owner) && (t.owner() != 0) && (t.armies() > strongestArmy)) {
        strongest = t.id();
        strongestArmy = t.armies();
      }
      border.avanzar();
    } while (! (border.finalLlista()));
    
    return strongestArmy;
  }
  
  /**
  * @return 0 si no hi ha frontera o l'id del territori amb menys armies enemigues
  */
  public int WeakestFront()  throws PDIdreta{
    Territory t;
    int strongest = 0;
    int strongestArmy = 999999999;

    if (border.buida()) {
      return 0;
    }
        
    border.principi();
    do {
      t = (Territory)border.actual();
      if ((t.owner() !=  owner) && (t.owner() != 0) && (t.armies() < strongestArmy)) {
        strongest = t.id();
        strongestArmy = t.armies();
      }
      border.avanzar();
    } while (! (border.finalLlista()));

    if (strongestArmy == 999999999 ) {
      return 0;
    } else {
      return strongest;
    }
  }
  
  /**
  * @return 0 si no hi ha frontera o el nombre d'armies del territori amb menys armies enemigues
  */
  public int WeakestFrontArmy()  throws PDIdreta{
    Territory t;
    int strongest = 0;
    int strongestArmy = 999999999;
    
    if (border.buida()) {
      return 0;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if ((t.owner() !=  owner) && (t.owner() != 0) && (t.armies() < strongestArmy)) {
        strongest = t.id();
        strongestArmy = t.armies();
      }
      border.avanzar();
    } while (! (border.finalLlista()));
        
    if (strongestArmy == 999999999 ) {
      return 0;
    } else {
      return strongestArmy;
    }
  }
  
  
  /**
  * @return Retorna el total de armies enemigues que rodejen el territori
  */
  public int Pressure()  throws PDIdreta{
    Territory t;
    int strongestArmy = 0;
    
    if (border.buida()) {
      return 0;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if ((t.owner() !=  owner) && (t.owner() != 0)) {
        strongestArmy += t.armies();
      }
      border.avanzar();
    } while (! (border.finalLlista()));
    
    return strongestArmy;
  }
  
  
  /**
  * Retorna l'id del territori numero "num" que fa frontera amb nosaltres
  * Retorna 0 si no existeix el territori num
  */
  public int Border(int num) throws PDIdreta{
    Territory t;
    int index = 0;
    
    if (border.buida()) {
      return 0;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if (this.borderWith(t.nom())) {
        index++;
        if (index==num) {
          return t.id();
        }
      }
      border.avanzar();
    } while (! (border.finalLlista()));
    
    return 0;
  }
  
  /**
  * Retorna el territori numero "num" que fa frontera amb nosaltres pero no es del nostre continent
  * Retorna null si no existeix el territori num
  * Util per Border de Continent (CBorder de Risk)
  */
  public Territory SpecialBorder(int num) throws PDIdreta{
    Territory t;
    int index = 0;
    
    if (border.buida()) {
      return null;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if ((this.borderWith(t.nom())) && (t.continentID != continentID)) {
        index++;
        if (index==num) {
          return t;
        }
      }
      border.avanzar();
    } while (! (border.finalLlista()));
    
    return null;
  }
  
  /**
  * Retorna el nombre de territoris que fan frontera amb nosaltres pero no es del nostre continent
  * Retorna null si no existeix el territori num
  * Util per Border de Continent (CBorder de Risk)
  */
  public int SpecialBorderCount() throws PDIdreta{
    Territory t;
    int index = 0;
    
    if (border.buida()) {
      return 0;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if ((this.borderWith(t.nom())) && (t.continentID != continentID)) {
        index++;
      }
      border.avanzar();
    } while (! (border.finalLlista()));
    
    return index;
  }
  
  
  /**
  * Retorna l'id del territori numero "num" que fa frontera amb nosaltres i es de l'enemic
  * Retorna 0 si no existeis el territori num
  */
  public int Front(int num) throws PDIdreta{
    Territory t;
    int index = 0;
    
    if (border.buida()) {
      return 0;
    }
    
    border.principi();
    do {
      t = (Territory)border.actual();
      if ((this.borderWith(t.nom())) && (t.owner != owner)) {
        index++;
        if (index==num) {
          return t.id();
        }
      }
      border.avanzar();
    } while (! (border.finalLlista()));
    
    return 0;
  }
  
  
  /**
  * @return Total de territoris que fan frontera amb aquest
  */
  public int borderTotal() {
    return border.elements();
  }
  
  
}
