package Risk;

import Risk.Internals.Mapa;
import Risk.Internals.Exceptions.*;

/**
 * Risk no es ms que una classe intermitja entre els Jugadors i el Mapa real.
 * Ha sigut necessaria la seva creaci per aix noms donar accs a la consulta
 * del mapa desde les classes Jugador, i no donar accs per modificar.
 * Aqui esta l'API real que pot utilitzar un jugador.
 */
public class Risk {
  private Mapa World;

  public Risk(Mapa m){
    World = m;
  }
  
  
  /***************************** CONSULTES SOBRE EL MAPA *****************************/
  /************ JUGADORS ***************/
  
  /**
  * Retorna l'id del jugador que t el torn.
  */
  public int PMe(){
    return World.torn();
  }
  
  /**
  * Retorna el nom del jugador amb l'id donat
  */
  public String PName(int id) throws JaExisteix, NoExisteix, PDIdreta {
    return World.PName(id);
  }
  
  /**
  * Retorna True si el jugador indicat esta actiu (t el torn)
  */
  public boolean PActive(int id){
    return World.PActive(id);
  }
  
  /**
  * Retorna el nombre d'armies que te el jugador indicat
  */
  public int PArmiesCount(int id) throws NoExisteix, PDIdreta{
    return World.PArmiesCount(id);
  }
  
  /**
  * Retorna el nombre d'armies del jugador indicat que encara ha de colocar
  */
  public int PNewArmies(int id){
    return World.PNewArmies(id);
  }
  
  /**
  * Retorna el nombre de territoris controlats pel jugador indicat.
  */
  public int PTerritories(int id) throws NoExisteix, PDIdreta{
    return World.PTerritories(id);
  }
  
  /**************** TERRITORIS ****************/
  
  /**
  * Retorna el nom del territori en la posici donada.
  * @param pos La posici del territori o el ID
  * @return Retorna el nomb del territori
  */
  public String TName(int id) throws PDIdreta, NoExisteix{
    return World.TName(id);
  }
  
  /**
  * Retorna l'id del propietari del territori indicat
  */
  public int TOwner(int id) throws PDIdreta, NoExisteix{
    return World.TOwner(id);
  }
  
  /**
  * Retorna el nombre d'armies del territori indicat
  */
  public int TArmies(int id) throws PDIdreta, NoExisteix{
    return World.TArmies(id);
  }
  
  /**
  * Retorna l'id del continent al qual pertany el territori indicat
  */
  public int TContinent(int id) throws PDIdreta, NoExisteix{
    return World.TContinent(id);
  }
  
  /**
  * Retorna el nombre de fronteres que te el territori indicat
  */
  public int TBordersCount(int id) throws PDIdreta, NoExisteix{
    return World.TBordersCount(id);
  }
  
  /**
  * Retorna l'id del territori numero "num" que fa frontera amb el territori indicat (id)
  */
  public int TBorder(int id, int num) throws PDIdreta, NoExisteix{
    return World.TBorder(id, num);
  }
  
  /**
  * Retorna true si id1 (territori) fa frontera amb id2 (territori)
  */
  public boolean TIsBordering(int id1, int id2) throws PDIdreta, NoExisteix{
    return World.TIsBordering(id1, id2);
  }
  
  /**
  * Retorna si el territori indicat fa frontera amb un territori enemic
  */
  public boolean TIsFront(int id) throws PDIdreta, NoExisteix{
    return World.TIsFront(id);
  }
  
  /**
  * Retorna True si el territori indicat ens pertany
  */
  public boolean TIsMine(int id) throws PDIdreta, NoExisteix{
    return World.TIsMine(id);
  }
  
  /**
  * Retorna el nombre de fronteres a territoris ocupas per enemics
  */
  public int TFrontsCount(int id) throws PDIdreta, NoExisteix{
    return World.TFrontsCount(id);
  }
  
  /**
  * Retorna l'id del territori numero "num" que fa frontera amb el territori indicat (id) i es de l'enemic
  */
  public int TFront(int id, int num) throws PDIdreta, NoExisteix{
    return World.TFront(id, num);
  }
  
  /**
  * Retorna la frontera ocupada per enemics mes forta
  */
  public int TStrongestFront(int id) throws PDIdreta, NoExisteix{
    return World.TStrongestFront(id);
  }
  
  /**
  * Retorna el nombre d'armis de la frontera ocupada per enemics mes forta
  */
  public int TStrongestFrontArmy(int id) throws PDIdreta, NoExisteix{
    return World.TStrongestFrontArmy(id);
  }
  
  /**
  * Retorna la frontera ocupada per enemics mes debil
  */
  public int TWeakestFront(int id) throws PDIdreta, NoExisteix{
    return World.TWeakestFront(id);
  }
  
  /**
  * Retorna el nombre d'armis de la frontera ocupada per enemics mes debil
  */
  public int TWeakestFrontArmy(int id) throws PDIdreta, NoExisteix{
    return World.TWeakestFrontArmy(id);
  }
  
  /**
  * Retorna el total d'armies del territoris que fan frontera amb l'indicat
  */
  public int TPressure(int id) throws PDIdreta, NoExisteix{
    return World.TPressure(id);
  }
  
  /**************** CONTINENTS ****************/
  
  /**
  * Retorna el nom del Continent en la posici donada.
  * @param pos La posici del Continent o el ID
  * @return Retorna el nomb del Continent
  */
  public String CName(int id) throws NoMemoria, PDIdreta, NoExisteix{
    return World.CName(id);
  }
  
  /**
  * @return Retorna 0 si te ms d'un ocupant, sino l'id de l'ocupant
  */
  public int COwner(int id) throws PDIdreta{
    return World.COwner(id);
  }
  
  /**
  * @param id ID del continent que volem saber quants territoris te
  * @return Retorna el nombre de territoris que pertanyen a un continent
  */
  public int CTerritoriesCount(int id) throws PDIdreta{
    return World.CTerritoriesCount(id);
  }
  
  
  /**
  * Retorna l'ID del territori numero "num" del continent indicat (id)
  */
  public int CTerritory(int id, int num) throws PDIdreta {
    return World.CTerritory(id, num);
  }
  
  
  /**
  * @param id ID del continent que volem saber quantes fronteres te
  * @return Retorna el nombre de fronteres d'un continent
  */
  public int CBordersCount(int id) throws PDIdreta, NoExisteix{
    return World.CBordersCount(id);
  }

  /**
  * Rertorna l'id del territori numero "num" que fa frontera amb el continent indicat (id)
  */
  public int CBorder(int id, int num) throws PDIdreta, NoExisteix {
    return World.CBorder(id, num);
  }
  
  /**************** STATUS DEL JOC ****************/
  /**
  * Retorna True si el jugador que te el torn (el que fa la crida) ha conquistat
  * algun territori durant el seu torn
  */
  public boolean SConquest() throws JaExisteix, NoExisteix, PDIdreta {
    return World.SConquest();
  }
  
  /**
  * Retorna el nombre de jugadors
  */
  public int SPlayersCount(){
    return World.SPlayersCount();
  }
  
  /***************************** FI CONSULTES SOBRE EL MAPA *****************************/
}
      

